// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "audiosystemopenal.h"
#include "vorbis_loader.h"
#include <stdlib.h>
#include <iostream>

#if defined(USE_SOUND)

#define GP(x)   alGetProcAddress((const ALubyte *) x)

#define DATABUFFERSIZE 10000000

/*
ALboolean   (*alCaptureInit)    ( ALenum format, ALuint rate, ALsizei bufferSize );
ALboolean   (*alCaptureDestroy) ( ALvoid );
ALboolean   (*alCaptureStart)   ( ALvoid );
ALboolean   (*alCaptureStop)    ( ALvoid );
ALsizei     (*alCaptureGetData) ( ALvoid* data, ALsizei n, ALenum format, ALuint rate );
*/

OpenAL::OpenAL()
{
}


OpenAL::~OpenAL()
{
}

int OpenAL::Init()
{
    // -----------------------------------------------------------------------------------
    // Initialise the device and context
    // -----------------------------------------------------------------------------------

    dev = alcOpenDevice( NULL );
    if( dev == NULL )
    {
        return 1;
    }

    context_id = alcCreateContext(dev, NULL);

    alcMakeContextCurrent(context_id);


    // -----------------------------------------------------------------------------------
    // Get pointers to some useful extensions.
    // -----------------------------------------------------------------------------------

/*
    Uncomment these if you get around to using them.

    alCaptureInit    = (ALboolean (*)( ALenum, ALuint, ALsizei )) GP("alCaptureInit_EXT");
    alCaptureDestroy = (ALboolean (*)( ALvoid )) GP("alCaptureDestroy_EXT");
    alCaptureStart   = (ALboolean (*)( ALvoid )) GP("alCaptureStart_EXT");
    alCaptureStop    = (ALboolean (*)( ALvoid )) GP("alCaptureStop_EXT");
    alCaptureGetData = (ALsizei (*)( ALvoid*, ALsizei, ALenum, ALuint )) GP("alCaptureGetData_EXT");

    if ( alCaptureInit == NULL || alCaptureDestroy == NULL || alCaptureStart == NULL ||
        alCaptureStop == NULL || alCaptureGetData == NULL)
    {
        std::cout << "Could not GetProcAddress alCaptureXXX_EXT" << std::endl;
        return 1;
    }
*/

    return 0;
}

int OpenAL::Shutdown()
{
    // -----------------------------------------------------------------------------------
    // Close the device and context
    // -----------------------------------------------------------------------------------

    // From OpenAL docs:
    // The correct way to destroy a context is to first release it using alcMakeCurrent
    // and NULL. Applications should not attempt to destroy a current context.

    alcMakeContextCurrent(NULL);

    alcDestroyContext(context_id);

    alcCloseDevice(dev);

    return 0;
}

void OpenAL::SetListenerPosition(vector3 pos)
{
    ALfloat position[] = { pos.x, pos.y, pos.z };

    alListenerfv(AL_POSITION, position );
}

void OpenAL::SetListenerVelocity(vector3 vel)
{
    ALfloat velocity[] = { vel.x, vel.y, vel.z };

    alListenerfv(AL_VELOCITY, velocity );
}

void OpenAL::SetListenerOrientation(vector3 lookat, vector3 up)
{
    ALfloat orientation[] = { lookat.x, lookat.y, lookat.z, up.x, up.y, up.z };

    alListenerfv(AL_ORIENTATION, orientation );
}

ALuint OpenAL::CreateBuffer()
{
    ALuint buffer;

    alGenBuffers(1, &buffer);

    return buffer;
}

ALuint OpenAL::CreateSource()
{
    ALuint source;

    alGenSources(1, &source);

    return source;
}

int OpenAL::LoadBuffer(ALuint buffer, std::string ogg_file)
{
    unsigned char* data = new unsigned char[DATABUFFERSIZE];

    int size = DATABUFFERSIZE;
    int frequency;
    bool stereo;

    load_vorbis(ogg_file, data, size, frequency, stereo);

    if (stereo == false)
    {
        alBufferData(buffer, AL_FORMAT_MONO16, data, size, frequency);
    }
    else
    {
        alBufferData(buffer, AL_FORMAT_STEREO16, data, size, frequency);
    }

    delete[] data;

    return 0;
}

bool OpenAL::SourceIsPlaying(ALuint source)
{
    ALint state;

    if(alIsSource(source) == AL_FALSE)
    {
        return false;
    }

    state = AL_INITIAL;
    alGetSourcei(source, AL_SOURCE_STATE, &state);
    switch(state)
    {
        case AL_PLAYING:
        case AL_PAUSED:
            return true;
        default:
            break;
    }

    return false;
}

void OpenAL::SetSourcePosition(ALuint source, vector3 pos)
{
    ALfloat position[] = { pos.x, pos.y, pos.z };

    alSourcefv(source, AL_POSITION, position);
}

void OpenAL::SetSourceVelocity(ALuint source, vector3 vel)
{
    ALfloat velocity[] = { vel.x, vel.y, vel.z };
    
    alSourcefv(source, AL_VELOCITY, velocity);
}

#endif // defined(USE_SOUND)




