// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "editor_pick.h"
#include <stdio.h>

// Returns the pick ray in 3D space corresponding to the mouse cursor.
void mouse_pick_ray(matrix44 projection, matrix44 view, int mouse_x, int mouse_y,
    int viewport_width, int viewport_height, vector3& ray_origin, vector3& ray_direction)
{
    // Compute the vector of the pick ray in screen space
    vector3 v;
    v.x =  (((2.0f * (float)mouse_x) / (float)viewport_width ) - 1.0f) / projection[0][0];
    v.y = -(((2.0f * (float)mouse_y) / (float)viewport_height) - 1.0f) / projection[1][1];
    v.z = -1.0f;

    // Get the inverse view matrix
    matrix44 m = InvertMatrix44(view);

    // Transform the screen space pick ray into 3D space
    ray_direction.x  = v.x*m[0][0] + v.y*m[1][0] + v.z*m[2][0];
    ray_direction.y  = v.x*m[0][1] + v.y*m[1][1] + v.z*m[2][1];
    ray_direction.z  = v.x*m[0][2] + v.y*m[1][2] + v.z*m[2][2];
    ray_origin.x     = m[3][0];
    ray_origin.y     = m[3][1];
    ray_origin.z     = m[3][2];
}
