// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef EDITOR_WORLD_H
#define EDITOR_WORLD_H

#include "level.h"

class EditorWorld
{
public:
    //! Constructor
    EditorWorld(Level* level, IGraphicsSystem* IGS);

    //! Destructor
    ~EditorWorld();

    // Create a new world
    void New();

    // Saves the world to a slot
    void Save(int number);

    // Loads the world from a slot
    void Load(int number);

    // Feeds the world editor the ray corresponding to the mouse cursor projected into 3D.
    void Ray(bool active, vector3 ray_origin, vector3 ray_destination, float horizontal_plane);

    void RenderObject();

private:
    Level* level;
    IGraphicsSystem* IGS;

    TEXTURE_HANDLE font_texture;

    std::vector<std::string> object_filenames;

    int current_object;
    float current_rotation;
    vector3 current_position;

    IModel* current_model;
};

#endif // EDITOR_WORLD_H

