// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "filenames.h"
#include <stdio.h>

std::string argv_zero;

bool file_exists(std::string filename)
{
    FILE* test;
    test = fopen(filename.c_str(), "rb");
    if (test == NULL)
    {
        return false;
    }

    fclose(test);

    return true;
}

// Returns the directory in which data is stored
std::string data_dir()
{
    static std::string dir;
    static std::string known_filename("detail.png");

    if (dir == "")
    {
        // Check if files are in arv[0]/data
        size_t i = argv_zero.rfind("/");
        //std::cout << argv_zero.substr(0, i) + slash() + "data" + slash() + known_filename << std::endl;
        if (file_exists(argv_zero.substr(0, i) + slash() + "data" + slash() + known_filename))
        {
            dir = argv_zero.substr(0, i) + slash() + "data";
        }

        i = argv_zero.substr(0, i).rfind("/");
        //std::cout << argv_zero.substr(0, i) + slash() + "data" + slash() + known_filename << std::endl;
        if (file_exists(argv_zero.substr(0, i) + slash() + "data" + slash() + known_filename))
        {
            dir = argv_zero.substr(0, i) + slash() + "data";
        }

        // Check if files are in data
        if (file_exists("data" + slash() + known_filename))
        {
            dir = "data";
        }
        // Check if files are in ../data
        if (file_exists(".." + slash() + "data" + slash() + known_filename))
        {
            dir = ".." + slash() + "data";
        }
#if defined(DATADIR)
        // Check if files are in DATADIR.
        if (file_exists(std::string(DATADIR) + slash() + known_filename))
        {
            dir = DATADIR;
        }
#endif //defined (DATADIR)
    }

    return dir;
}

// Returns the appropriate slash symbol e.g. / or \\.
std::string slash()
{
#if defined(WIN32)
    return "\\";
#else
    return "/";
#endif
}

// Call this function with the value of argv[0]
void set_argv_zero(std::string argv)
{
    argv_zero = argv;
}

std::string remove_dot_slash(std::string filename)
{
    std::string result;

    for (unsigned int c = 0; c < filename.size(); c++)
    {
        if (filename[c] == '.' || filename[c] == '\\')
        {
            if (filename[c] == '.')
            {
                if ((c + 1) < filename.size())
                {
                     if (filename[c+1] != '\\')
                     {
                        result += filename[c];
                     }
                }
            }
            else // if (filename[c] == '\\')
            {
                if (c > 0)
                {
                     if (filename[c-1] != '.')
                     {
                        result += filename[c];
                     }
                }
            }
        }
        else
        {
            result += filename[c];
        }
    }
    return result;
}

