// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "font.h"
#include "hud_renderer.h"
#include <iostream>

// Font width in pixels
#define FONT_SIZE 1024

// Created by FontMapper v0.01
// TextMapper (c) 2001 Linus Tan

const int _Arial_Black_Widths[] = {
 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36,
 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36,
 16, 16, 24, 32, 32, 48, 43, 13, 19, 19, 27, 32, 16, 16, 16, 13,
 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 16, 16, 32, 32, 32, 29,
 36, 37, 37, 37, 37, 35, 32, 40, 40, 19, 32, 40, 32, 45, 40, 40,
 35, 40, 37, 35, 35, 40, 37, 48, 37, 37, 35, 19, 13, 19, 32, 24,
 16, 32, 32, 32, 32, 32, 19, 32, 32, 16, 16, 32, 16, 48, 32, 32,
 32, 32, 21, 29, 21, 32, 29, 45, 32, 29, 27, 19, 13, 19, 32, 36,
 32, 36, 13, 32, 24, 48, 32, 32, 16, 48, 35, 16, 48, 36, 35, 36,
 36, 13, 13, 24, 24, 24, 24, 48, 16, 46, 29, 16, 48, 36, 27, 37,
 16, 16, 32, 32, 32, 32, 13, 32, 16, 38, 19, 32, 32, 16, 38, 24,
 19, 32, 19, 19, 16, 32, 41, 16, 16, 19, 19, 32, 48, 48, 48, 29,
 37, 37, 37, 37, 37, 37, 48, 37, 35, 35, 35, 35, 19, 19, 19, 19,
 37, 40, 40, 40, 40, 40, 40, 32, 40, 40, 40, 40, 40, 37, 35, 32,
 32, 32, 32, 32, 32, 32, 48, 32, 32, 32, 32, 32, 16, 16, 16, 16,
 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 29, 32, 29,
};

const unsigned int _Verdana_Widths[] = {
 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13,
 5, 5, 5, 10, 8, 13, 9, 3, 6, 6, 9, 9, 5, 7, 5, 6,
 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 6, 6, 9, 9, 9, 7,
 13, 9, 8, 9, 9, 8, 8, 9, 9, 5, 6, 8, 7, 11, 9, 10,
 8, 10, 8, 9, 9, 9, 9, 13, 9, 9, 9, 6, 6, 6, 11, 8,
 8, 8, 8, 8, 8, 8, 5, 8, 8, 3, 4, 7, 3, 11, 8, 8,
 8, 8, 5, 7, 6, 8, 8, 11, 7, 8, 7, 8, 7, 8, 11, 13,
 8, 13, 3, 8, 6, 11, 8, 8, 8, 20, 9, 6, 14, 13, 9, 13,
 13, 3, 3, 6, 6, 7, 8, 13, 8, 13, 7, 6, 13, 13, 7, 9,
 5, 5, 8, 8, 8, 8, 7, 8, 8, 13, 7, 8, 9, 7, 13, 8,
 7, 9, 7, 7, 8, 8, 8, 5, 8, 7, 7, 8, 13, 13, 13, 7,
 9, 9, 9, 9, 9, 9, 12, 9, 8, 8, 8, 8, 5, 5, 5, 5,
 9, 9, 10, 10, 10, 10, 10, 11, 10, 9, 9, 9, 9, 9, 8, 8,
 8, 8, 8, 8, 8, 8, 11, 8, 8, 8, 8, 8, 3, 3, 3, 3,
 8, 8, 8, 8, 8, 8, 8, 9, 8, 8, 8, 8, 8, 8, 8, 8,
};

const unsigned int _VerdanaBold_Widths[] = {
 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13,
 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13,
 4, 6, 7, 11, 9, 17, 11, 4, 7, 7, 9, 11, 5, 7, 5, 9,
 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 5, 5, 11, 11, 11, 8,
 13, 10, 10, 10, 10, 9, 9, 10, 11, 6, 7, 9, 8, 12, 10, 11,
 9, 11, 9, 9, 8, 10, 10, 14, 10, 10, 9, 6, 9, 6, 10, 9,
 9, 9, 9, 8, 9, 9, 5, 9, 9, 4, 5, 8, 4, 14, 9, 9,
 9, 9, 6, 8, 6, 9, 9, 12, 9, 9, 8, 9, 8, 9, 11, 13,
 10, 13, 4, 9, 8, 15, 10, 10, 9, 24, 9, 7, 15, 13, 9, 13,
 13, 4, 4, 8, 8, 10, 9, 13, 9, 13, 8, 7, 14, 13, 8, 10,
 4, 6, 9, 9, 9, 9, 8, 9, 9, 12, 8, 11, 11, 7, 12, 9,
 7, 11, 8, 8, 9, 9, 9, 5, 9, 8, 8, 11, 15, 15, 15, 8,
 10, 10, 10, 10, 10, 10, 14, 10, 9, 9, 9, 9, 6, 6, 6, 6,
 10, 10, 11, 11, 11, 11, 11, 11, 11, 10, 10, 10, 10, 10, 9, 9,
 9, 9, 9, 9, 9, 9, 14, 8, 9, 9, 9, 9, 4, 4, 4, 4,
 9, 9, 9, 9, 9, 9, 9, 11, 9, 9, 9, 9, 9, 9, 9, 9,
};

// Extended width info only available with TrueType fonts!

// WidthLeading: Specifies the distance to add to the
// current position before drawing the character glyph.

// WidthGlyph: Specifies the width of the drawn portion of
// the character glyph.

// WidthTrailing: Specifies the current position to provide
// white space to the right of the character glyph.

const int _Arial_Black_WidthLeading[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 3, 1, 1, 2, 2, 2, 2, 3, 1, 5, 3, 3, 1, 3, 0,
 2, 4, 2, 2, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 2,
 0, 0, 4, 2, 4, 4, 4, 2, 4, 4, 1, 4, 4, 4, 4, 2,
 4, 2, 4, 1, 1, 4, 0, 0, 0, 0, 2, 2, 0, 1, 3, 0,
 0, 2, 3, 2, 2, 2, 0, 2, 3, 3,-3, 3, 3, 3, 3, 2,
 3, 2, 3, 1, 1, 3, 0, 0, 1, 0, 2, 0, 4, 1, 2, 0,
-1, 0, 2, 1, 2, 3, 3, 3,-1, 0, 1, 1, 2, 0, 2, 0,
 0, 2, 2, 2, 2, 4, 0, 0, 0, 0, 1, 1, 2, 0, 2, 0,
 0, 3, 2, 3, 2, 0, 4, 2, 0, 1, 1, 3, 3, 1, 1, 0,
 3, 3, 1, 1, 4, 3, 3, 3, 0, 4, 1, 2, 4, 4, 2, 1,
 0, 0, 0, 0, 0, 0,-2, 2, 4, 4, 4, 4, 2, 4, 1, 2,
 0, 4, 2, 2, 2, 2, 2, 3, 0, 4, 4, 4, 4, 0, 4, 3,
 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 3,-1, 0,
 2, 3, 2, 2, 2, 2, 2, 3, 2, 3, 3, 3, 3, 0, 3, 0,
};

const int _Arial_Black_WidthGlyph[] = {
 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
 1, 10, 22, 30, 29, 44, 39, 10, 15, 15, 18, 26, 10, 14, 10, 13,
 28, 20, 28, 28, 31, 28, 28, 28, 28, 28, 10, 10, 26, 26, 26, 26,
 36, 37, 31, 33, 31, 29, 26, 36, 32, 11, 27, 36, 26, 37, 32, 36,
 29, 37, 34, 32, 33, 32, 37, 49, 37, 37, 31, 16, 13, 16, 26, 24,
 12, 29, 27, 28, 27, 28, 21, 27, 26, 10, 16, 29, 10, 42, 26, 28,
 27, 27, 20, 26, 20, 26, 29, 45, 31, 30, 23, 18, 6, 18, 27, 24,
 32, 24, 10, 31, 22, 42, 26, 26, 18, 48, 32, 14, 44, 24, 31, 24,
 24, 10, 10, 22, 22, 16, 24, 48, 16, 44, 26, 14, 44, 24, 23, 37,
 1, 10, 28, 29, 27, 33, 6, 28, 16, 36, 17, 27, 26, 14, 36, 24,
 13, 26, 18, 18, 12, 26, 35, 10, 15, 11, 18, 27, 43, 43, 45, 26,
 37, 37, 37, 37, 37, 37, 48, 33, 29, 29, 29, 29, 13, 14, 18, 16,
 35, 32, 36, 36, 36, 36, 36, 26, 40, 32, 32, 32, 32, 37, 29, 27,
 29, 29, 29, 29, 29, 29, 44, 28, 28, 28, 28, 28, 13, 13, 18, 16,
 28, 26, 28, 28, 28, 28, 28, 26, 28, 26, 26, 26, 26, 30, 27, 30,
};

const int _Arial_Black_WidthTrailing[] = {
 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
 15, 3, 1, 1, 1, 2, 2, 1, 1, 3, 4, 3, 3, 1, 3, 0,
 2, 8, 2, 2, 0, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 1,
 0, 0, 2, 2, 2, 2, 2, 2, 4, 4, 4, 0, 2, 4, 4, 2,
 2, 1,-1, 2, 1, 4, 0,-1, 0, 0, 2, 1, 0, 2, 3, 0,
 4, 1, 2, 2, 3, 2,-2, 3, 3, 3, 3, 0, 3, 3, 3, 2,
 2, 3,-2, 2, 0, 3, 0, 0, 0,-1, 2, 1, 3, 0, 3, 12,
 1, 12, 1, 0, 0, 3, 3, 3,-1, 0, 2, 1, 2, 12, 2, 12,
 12, 1, 1, 0, 0, 4, 0, 0, 0, 2, 2, 1, 2, 12, 2, 0,
 15, 3, 2, 0, 3,-1, 3, 2, 0, 1, 1, 2, 3, 1, 1, 0,
 3, 3, 0, 0, 0, 3, 3, 3, 1, 4, 0, 3, 1, 1, 1, 2,
 0, 0, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2, 4, 1, 0, 1,
 2, 4, 2, 2, 2, 2, 2, 3, 0, 4, 4, 4, 4, 0, 2, 2,
 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 3, 0,-1, 0,
 2, 3, 2, 2, 2, 2, 2, 3, 2, 3, 3, 3, 3,-1, 2,-1,
};


const int _VerdanaBold_WidthLeading[] = {
 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
 0, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 0, 1, 2, 1,
 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 2, 2, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0,
 2, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 3, 1, 1, 2,
 0, 2, 0, 1, 0, 1, 2, 2, 2, 1, 1, 1, 1, 2, 1, 2,
 2, 0, 1, 0, 1, 2, 1, 1, 2, 0, 1, 2, 1, 2, 1, 1,
 0, 2, 1, 1, 1, 0, 3, 1, 2, 1, 1, 1, 2, 1, 1, 0,
 1, 2, 1, 1, 3, 1, 1, 2, 3, 2, 1, 2, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 0, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0,
 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1,
};


const int _VerdanaBold_WidthGlyph[] = {
 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
 1, 2, 5, 8, 7, 15, 9, 2, 5, 5, 7, 7, 4, 5, 2, 7,
 7, 6, 7, 7, 8, 7, 7, 7, 7, 7, 2, 4, 8, 7, 8, 6,
 11, 8, 8, 8, 8, 7, 7, 8, 9, 4, 6, 8, 7, 10, 8, 9,
 7, 9, 8, 7, 8, 8, 8, 12, 8, 8, 7, 4, 7, 4, 8, 9,
 3, 7, 7, 7, 7, 7, 5, 7, 7, 2, 4, 7, 2, 12, 7, 7,
 7, 7, 5, 6, 5, 7, 7, 10, 7, 7, 6, 7, 2, 7, 9, 10,
 9, 10, 3, 8, 7, 12, 6, 6, 5, 22, 7, 4, 13, 10, 7, 10,
 10, 3, 3, 7, 7, 6, 7, 11, 6, 12, 6, 4, 12, 10, 6, 8,
 1, 2, 7, 7, 7, 8, 2, 7, 4, 10, 6, 8, 7, 5, 10, 9,
 5, 7, 5, 5, 3, 7, 6, 2, 4, 4, 6, 8, 13, 13, 13, 6,
 8, 8, 8, 8, 8, 8, 12, 8, 7, 7, 7, 7, 4, 4, 5, 4,
 9, 8, 9, 9, 9, 9, 9, 7, 9, 8, 8, 8, 8, 8, 7, 7,
 7, 7, 7, 7, 7, 7, 12, 7, 7, 7, 7, 7, 3, 3, 4, 4,
 7, 7, 7, 7, 7, 7, 7, 8, 7, 7, 7, 7, 7, 7, 7, 7,
};

const int _VerdanaBold_WidthTrailing[] = {
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 3, 2, 1, 2, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1,
 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 2, 2, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1,
 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0,
 4, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1,
 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 1, 1, 1,
 1, 1, 1, 0, 1, 2, 2, 2, 2, 1, 1, 2, 1, 1, 1, 1,
 1, 1, 0, 1, 0, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 3, 2, 1, 1, 1, 1, 3, 1, 3, 1, 1, 2, 2, 1, 1, 0,
 1, 2, 2, 2, 3, 1, 2, 1, 2, 2, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1,
 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
};


const int _Verdana_WidthLeading[] = {
 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
 0, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 2, 0,
 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 2, 0, 1, 1, 0,
 2, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 3, 2, 1, 2,
 0, 2, 0, 0, 0, 1, 1, 1, 2, 1, 1, 1, 1, 2, 1, 2,
 2, 0, 1, 0, 1, 1, 1, 1, 2, 2, 1, 1, 1, 2, 1, 1,
 0, 2, 1, 1, 1, 1, 3, 1, 2, 1, 1, 1, 1, 1, 1, 0,
 1, 1, 1, 1, 3, 1, 1, 2, 2, 2, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0,
 0, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0,
};

const int _Verdana_WidthGlyph[] = {
 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
 1, 1, 3, 8, 6, 11, 9, 1, 4, 4, 5, 7, 3, 5, 1, 6,
 6, 5, 6, 6, 7, 6, 6, 6, 6, 6, 1, 3, 7, 7, 7, 5,
 11, 7, 6, 7, 7, 6, 6, 7, 7, 3, 5, 6, 6, 9, 7, 8,
 6, 8, 8, 7, 7, 7, 9, 13, 9, 7, 7, 3, 6, 3, 9, 8,
 3, 6, 6, 6, 6, 6, 5, 6, 6, 1, 4, 6, 1, 9, 6, 6,
 6, 6, 4, 5, 4, 6, 6, 9, 7, 8, 5, 5, 1, 5, 8, 10,
 7, 10, 3, 8, 6, 9, 7, 7, 4, 17, 7, 3, 12, 10, 7, 10,
 10, 3, 3, 6, 6, 5, 6, 11, 5, 9, 5, 3, 11, 10, 5, 7,
 1, 1, 6, 6, 6, 7, 1, 6, 4, 11, 5, 6, 7, 5, 11, 8,
 5, 7, 4, 5, 3, 6, 6, 1, 3, 3, 5, 6, 11, 11, 11, 5,
 7, 7, 7, 7, 7, 7, 11, 7, 6, 6, 6, 6, 3, 4, 4, 5,
 8, 7, 8, 8, 8, 8, 8, 7, 8, 7, 7, 7, 7, 7, 6, 6,
 6, 6, 6, 6, 6, 6, 9, 6, 6, 6, 6, 6, 3, 3, 3, 3,
 6, 6, 6, 6, 6, 6, 6, 7, 6, 6, 6, 6, 6, 8, 6, 8,
};

const int _Verdana_WidthTrailing[] = {
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 4, 2, 1, 1, 1, 1,-1, 1, 1, 1, 2, 1, 1, 1, 2, 0,
 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 3, 2, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1,
 1, 1,-1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 2, 1, 0,
 3, 1, 1, 1, 1, 1,-1, 1, 1, 1, 1, 0, 1, 1, 1, 1,
 1, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 2, 3, 1, 2, 1,
 1, 1, 0, 0, 0, 1, 0, 0, 2, 2, 1, 2, 1, 1, 1, 1,
 1, 0,-1, 0,-1, 1, 1, 1, 1, 2, 1, 2, 1, 1, 1, 1,
 4, 2, 1, 1, 1, 0, 3, 1, 2, 1, 1, 1, 1, 1, 1, 0,
 1, 1, 2, 1, 2, 1, 1, 2, 3, 2, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0,
 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0,-1, 0, 0,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0,
};


void GetCharLocation(int& x, int& y, unsigned char c)
{
    x = c%16;
    y = c/16;
}

int GetFontTextureSize(FONT_TYPE typeface)
{
    if (typeface == ARIAL_BLACK)        return 1024;
    else if (typeface == VERDANA)       return 256;
    else /* typeface == VERDANA_BOLD */ return 256;
}

int GetCharSize(FONT_TYPE typeface)
{
    if (typeface == ARIAL_BLACK)        return 64;
    else if (typeface == VERDANA)       return 16;
    else /* typeface == VERDANA_BOLD */ return 16;
}

int GetWidthLeading(FONT_TYPE typeface, char ch)
{
    if (typeface == ARIAL_BLACK)        return _Arial_Black_WidthLeading[ch];
    else if (typeface == VERDANA)       return _Verdana_WidthLeading[ch];
    else /* typeface == VERDANA_BOLD */ return _VerdanaBold_WidthLeading[ch];
}

int GetWidthGlyph(FONT_TYPE typeface, char ch)
{
    if (typeface == ARIAL_BLACK)        return _Arial_Black_WidthGlyph[ch];
    else if (typeface == VERDANA)       return _Verdana_WidthGlyph[ch];
    else /* typeface == VERDANA_BOLD */ return _VerdanaBold_WidthGlyph[ch];
}

int GetWidthTrailing(FONT_TYPE typeface, char ch)
{
    if (typeface == ARIAL_BLACK)        return _Arial_Black_WidthTrailing[ch];
    else if (typeface == VERDANA)       return _Verdana_WidthTrailing[ch];
    else /* typeface == VERDANA_BOLD */ return _VerdanaBold_WidthTrailing[ch];
}

void GetCharLocation(FONT_TYPE typeface, float& x1, float& y1, float& x2, float& y2, unsigned char c)
{
    int x, y;

    GetCharLocation(x, y, c);
    
    float pixel_size = 1.0f / float(GetFontTextureSize(typeface));

    x1 = (GetCharSize(typeface) * x) * pixel_size;
    y1 = (GetCharSize(typeface) * y) * pixel_size;
    x2 = (GetCharSize(typeface) * (x + 1)) * pixel_size;
    y2 = (GetCharSize(typeface) * (y + 1)) * pixel_size;
}

void DisplayText(FONT_TYPE typeface, int x, int y, float size, std::string text,
    TEXTURE_HANDLE font_texture, IGraphicsSystem *IGS, COLOUR c)
{
    IGS->SetTexture(0, font_texture);

    float x1, y1, x2, y2;

    for (unsigned int ch = 0; ch < text.size(); ch++)
    {
        GetCharLocation(typeface, x1, y1, x2, y2, text[ch]);

        x += int(float(GetWidthLeading(typeface, (unsigned char)text[ch])) * size);

        HUD_Renderer::Instance().Draw2D(IGS, x, y, x + GetCharSize(typeface) * size, y + GetCharSize(typeface) * size,
            x1, y1, x2, y2, c);

        x += int(float(GetWidthGlyph(typeface, (unsigned char)text[ch]) +
            GetWidthTrailing(typeface, (unsigned char)text[ch]) + 1) * size);
    }
}

float GetTextWidth(FONT_TYPE typeface, float size, std::string text)
{
    float w = 0.0f;

    for (unsigned int ch = 0; ch < text.size(); ch++)
    {
        w += float(GetWidthLeading(typeface, (unsigned char)text[ch]) +
                    GetWidthGlyph(typeface, (unsigned char)text[ch]) +
                    GetWidthTrailing(typeface, (unsigned char)text[ch]) + 1) * size;
    }

    return w;
}

// Returns a string truncated to a certain width
std::string FitString(FONT_TYPE typeface, int width, float size, std::string text)
{
    int w = 0;
    
    std::string return_string;

    for (unsigned int ch = 0; ch < text.size(); ch++)
    {
        int char_width = int(float(GetWidthLeading(typeface, (unsigned char)text[ch]) +
                        GetWidthGlyph(typeface, (unsigned char)text[ch])) * size);

        if (w + char_width < width)
        {
            return_string += text[ch];
        }
        
        w += char_width;
        w += int(float(GetWidthTrailing(typeface, (unsigned char)text[ch]) + 1) * size);
    }

    return return_string;
}








