// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef GET_DISTANCE_H
#define GET_DISTANCE_H

#include "mtxlib.h"

inline float get_distance(const float& x1, const float& y1, const float& z1, const float& x2,
    const float& y2, const float& z2)
{   
    return (float)sqrt( ((x1-x2)*(x1-x2)) + ((y1-y2)*(y1-y2)) + ((z1-z2)*(z1-z2)) );
}

inline float get_distance(const vector3& v1, const vector3& v2)
{
    return (float)sqrt( ((v1.x-v2.x)*(v1.x-v2.x)) + ((v1.y-v2.y)*(v1.y-v2.y))
        + ((v1.z-v2.z)*(v1.z-v2.z)) );
}

inline float get_distance(const vector3& v, const float& x, const float& y, const float& z)
{
    return (float)sqrt( ((v.x-x)*(v.x-x)) + ((v.y-y)*(v.y-y)) + ((v.z-z)*(v.z-z)) );
}

inline double get_distance(const double& x1, const double& y1, const double& z1, const double& x2,
    const double& y2, const double& z2)
{
    return (float)sqrt( ((x1-x2)*(x1-x2)) + ((y1-y2)*(y1-y2)) + ((z1-z2)*(z1-z2)) );
}

inline double get_distance(const vector3& v, const double& x, const double& y, const double& z)
{
    return (float)sqrt( ((v.x-x)*(v.x-x)) + ((v.y-y)*(v.y-y)) + ((v.z-z)*(v.z-z)) );
}

#endif // GET_DISTANCE_H
