// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "hud_renderer.h"
#include "hud.h"
#include "font.h"

void draw_health_bar(IGraphicsSystem* IGS, int num_bars, int num_filled, HUD_COLOUR colour, float x_offset, float y_offset)
{
    int x;

    if (colour == BAR_RED)
    {
        for (x = 0; x < num_filled; x++)
        {
            HUD_Renderer::Instance().Draw2D(IGS, x_offset + x * 10.0, y_offset, x_offset+ (x+1) * 10.0, y_offset + 16.0,
                0.0 / 256.0, 0.0 / 256.0, 9.0 / 256.0, 16.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));
        }

        for (x = num_filled; x < num_bars; x++)
        {
            HUD_Renderer::Instance().Draw2D(IGS, x_offset + x * 10.0, y_offset, x_offset+ (x+1) * 10.0, y_offset + 16.0,
                10.0 / 256.0, 0.0 / 256.0, 20.0 / 256.0, 16.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));
        }
    }
    else // (colour == BAR_BLUE)
    {
        for (x = 0; x < num_filled; x++)
        {
            HUD_Renderer::Instance().Draw2D(IGS, x_offset + x * 10.0, y_offset, x_offset+ (x+1) * 10.0, y_offset + 16.0,
                32.0 / 256.0, 0.0 / 256.0, 41.0 / 256.0, 16.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));
        }

        for (x = num_filled; x < num_bars; x++)
        {
            HUD_Renderer::Instance().Draw2D(IGS, x_offset + x * 10.0, y_offset, x_offset + (x+1) * 10.0, y_offset + 16.0,
                42.0 / 256.0, 0.0 / 256.0, 52.0 / 256.0, 16.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));
        }
    }
}

void draw_message(std::string message_text, TEXTURE_HANDLE font_texture, IGraphicsSystem* IGS, float time)
{
    if (time < 500)
    {
        // 400 - 60 to 400 to 400 + 60
        // 600 to 500
        float y = 600.0f - time / 5.0;

        HUD_Renderer::Instance().Draw2D(IGS, 340.0, y, 400.0, y + 21.0,
            128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));

        HUD_Renderer::Instance().Draw2D(IGS, 400.0, y, 460.0, y + 21.0,
            128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));
    }
    else if (time < 5500)
    {
       // 400 - 60 to 400 to 400 + 60

        float y = 500.0;
        int max_width = GetTextWidth(ARIAL_BLACK, 0.4f, message_text) + 20.0;

        time -= 500;
        if (time > 1000) time = 1000;

        int x_offset = ((max_width / 2) * time) / 1000;

        HUD_Renderer::Instance().Draw2D(IGS, 340.0 - x_offset, y, 400.0 - x_offset, y + 21.0,
            128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));

        HUD_Renderer::Instance().Draw2D(IGS, 400.0 + x_offset, y, 460.0 + x_offset, y + 21.0,
            128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));

        float width_available = (max_width * time) / 1000 - 20.0;

        std::string fitted_string = FitString(ARIAL_BLACK, width_available, 0.4, message_text);

        DisplayText(ARIAL_BLACK, 400 - x_offset + 10, 495, 0.4, fitted_string,
            font_texture, IGS, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));
    }
    else if (time < 6500)
    {
        float y = 500.0;
        int max_width = GetTextWidth(ARIAL_BLACK, 0.4f, message_text) + 20.0;

        time -= 5500;
        if (time > 1000) time = 1000;
        time = 1000 - time;

        int x_offset = ((max_width / 2) * time) / 1000;

        HUD_Renderer::Instance().Draw2D(IGS, 340.0 - x_offset, y, 400.0 - x_offset, y + 21.0,
            128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));

        HUD_Renderer::Instance().Draw2D(IGS, 400.0 + x_offset, y, 460.0 + x_offset, y + 21.0,
            128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));

        float width_available = (max_width * time) / 1000 - 20.0;

        std::string fitted_string = FitString(ARIAL_BLACK, width_available, 0.4, message_text);

        DisplayText(ARIAL_BLACK, 400 - x_offset + 10, 495, 0.4, fitted_string,
            font_texture, IGS, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));
    }
    if (time < 7000)
    {
        // 400 - 60 to 400 to 400 + 60
        // 600 to 500
        time -= 6500;
        float y = 500.0f + time / 5.0;

        HUD_Renderer::Instance().Draw2D(IGS, 340.0, y, 400.0, y + 21.0,
            128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));

        HUD_Renderer::Instance().Draw2D(IGS, 400.0, y, 460.0, y + 21.0,
            128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));
    }
/*
    if (time < 500)
    {
        // 400 - 60 to 400 to 400 + 60
        // 600 to 500
        float y = 600.0f - time / 5.0;

        HUD_Renderer::Instance().Draw2D(IGS, 340.0, y, 400.0, y + 21.0,
            128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));

        HUD_Renderer::Instance().Draw2D(IGS, 400.0, y, 460.0, y + 21.0,
            128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));
    }
    else if (time < 6000)
    {
        // 400 - 60 to 400 to 400 + 60

        float y = 500.0;
        int max_width = GetTextWidth(ARIAL_BLACK, 0.4f, message_text) + 20.0;

        if (time < 5000)
        {
            time -= 500;
            if (time > 1000) time = 1000;

            int x_offset = ((max_width / 2) * time) / 1000;

            HUD_Renderer::Instance().Draw2D(IGS, 340.0 - x_offset, y, 400.0 - x_offset, y + 21.0,
                128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));

            HUD_Renderer::Instance().Draw2D(IGS, 400.0 + x_offset, y, 460.0 + x_offset, y + 21.0,
                128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));

            float width_available = (max_width * time) / 1000 - 20.0;

            if (time == 1000)
            {
                DisplayText(ARIAL_BLACK, 400 - x_offset + 10, 495, 0.4, message_text,
                    font_texture, IGS, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));
            }
            else
            {
                std::string fitted_string = FitString(ARIAL_BLACK, width_available, 0.4, message_text);

                DisplayText(ARIAL_BLACK, 400 - x_offset + 10, 495, 0.4, fitted_string,
                    font_texture, IGS, COLOUR(1.0f, 1.0f, 1.0f, 1.0f));
            }
        }
        else if (time < 6000)
        {
            time -= 5000;

            int x_offset = max_width / 2;
            float fade = 1.0 - (time / 1000.0);

            HUD_Renderer::Instance().Draw2D(IGS, 340.0 - x_offset, y, 400.0 - x_offset, y + 21.0,
                128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, fade));

            HUD_Renderer::Instance().Draw2D(IGS, 400.0 + x_offset, y, 460.0 + x_offset, y + 21.0,
                128.0 / 256.0, 0.0 / 256.0, 188.0 / 256.0, 21.0 / 256.0, COLOUR(1.0f, 1.0f, 1.0f, fade));

            DisplayText(ARIAL_BLACK, 400 - x_offset + 10, 495, 0.4, message_text,
                    font_texture, IGS, COLOUR(1.0f, 1.0f, 1.0f, fade));
        }
    }
*/
}
