// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "hud_renderer.h"

HUD_Renderer::HUD_Renderer()
{

}

HUD_Renderer::~HUD_Renderer()
{
}

void HUD_Renderer::Draw2D(IGraphicsSystem* IGS, float left_dest, float top_dest, float right_dest, float bottom_dest,
            float left_src, float top_src, float right_src, float bottom_src , COLOUR c)
{
    IGS->SetCullMode(CULL_NONE);
    IGS->EnableLighting(false);

    VERTEX_XYZ_UV1 p1, p2, p3, p4;
    p1.point.x = left_dest;
    p1.point.y = top_dest;
    p2.point.x = right_dest;
    p2.point.y = top_dest;
    p3.point.x = right_dest;
    p3.point.y = bottom_dest;
    p4.point.x = left_dest;
    p4.point.y = bottom_dest;

    p1.u = left_src;
    p1.v = top_src;
    p2.u = right_src;
    p2.v = top_src;
    p3.u = right_src;
    p3.v = bottom_src;
    p4.u = left_src;
    p4.v = bottom_src;

    IGS->Draw2D(p1, p2, p3, p4, c);
}

void HUD_Renderer::Draw2D(IGraphicsSystem* IGS, vector2 top_left, vector2 top_right, vector2 bottom_right,
    vector2 bottom_left, float left_src, float top_src, float right_src, float bottom_src , COLOUR c)
{
    IGS->SetCullMode(CULL_NONE);
    IGS->EnableLighting(false);

    VERTEX_XYZ_UV1 p1, p2, p3, p4;
    p1.point = top_left;
    p2.point = top_right;
    p3.point = bottom_right;
    p4.point = bottom_left;

    p1.u = left_src;
    p1.v = top_src;
    p2.u = right_src;
    p2.v = top_src;
    p3.u = right_src;
    p3.v = bottom_src;
    p4.u = left_src;
    p4.v = bottom_src;

    IGS->Draw2D(p1, p2, p3, p4, c);
}
