// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef HUD_RENDERER_H
#define HUD_RENDERER_H

#include "graphicssystem.h"

//! Class that deals with rendering 2D sprites overlayed on the 3D viewport.
class HUD_Renderer
{
public:
    //! Returns a reference to the one and only instance of this class.
    static HUD_Renderer& Instance()
    {
        static HUD_Renderer Obj;
        return Obj;
    }

    ~HUD_Renderer();

    // Draws a sprite. The texture to use must be set already before calling this function.
    void Draw2D(IGraphicsSystem* IGS, float left_dest, float top_dest, float right_dest, float bottom_dest,
        float left_src, float top_src, float right_src, float bottom_src , COLOUR c);

    // Draws a sprite. The texture to use must be set already before calling this function.
    // This version allows the sprite to be rotated.
    void Draw2D(IGraphicsSystem* IGS, vector2 top_left, vector2 top_right, vector2 bottom_right, vector2 bottom_left,
        float left_src, float top_src, float right_src, float bottom_src , COLOUR c);

private:

    // Constructor is private in this singleton
    HUD_Renderer();
};

#endif // HUD_RENDERER_H

