// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef MODEL_H
#define MODEL_H

#include "mtxlib.h"
#include "graphicssystem.h"
#include "shadowable.h"

/*!
    \file model.h
    \brief File contains abstract interface for a generic 3D model format.
*/

//! Abstract interface for a 3D Model.
class IModel : public Shadowable
{
public:
    //! Initialises the model.
    IModel() {};

    //! Deinitialises the model.
    virtual ~IModel() {};

    //! Loads a model from file.
    // directory is a string that is prepended to the filename and the filenames of all the textures
    virtual void Load(std::string directory, std::string file_name, IGraphicsSystem *IGS) = 0;

    //! Creates an animation instance for an animated model.
    virtual ANIMATION_INSTANCE_HANDLE CreateAnimationInstance() = 0;

    //! Deletes an animation instance.
    virtual void DeleteAnimationInstance(ANIMATION_INSTANCE_HANDLE instance) = 0;

    //! Updates an animated model
    virtual void Update(float time, ANIMATION_INSTANCE_HANDLE handle) = 0;

    //! Returns the radius and centre of the bounding sphere
    virtual void GetBoundingSphere(float& radius, vector3& centre) = 0;

protected:

    // Pointer to renderer interface
    IGraphicsSystem* IGS;
};

#endif // MODEL_H

