// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef MOUSE_H
#define MOUSE_H

/*!
    \file mouse.h
    \brief Handles the mouse.
*/

#include "SDL.h"

//! Class that handles input from the mouse.
class Mouse
{
public:
    //! Returns a reference to the one and only insatce of this class.
    static Mouse& Instance()
    {
        static Mouse Obj;
        return Obj;
    }

    //! Returns true if the left mouse button is down
    bool isLeftButtonDown();

    //! Returns true if the right mouse button is down
    bool isRightButtonDown();

    //! Returns the mouse state relative to the last call to this function.
    void getRelativeMouseState(int* x, int* y);

    //! Returns the absolute mouse state
    void getAbsoluteMouseState(int* x, int* y);

    //! Returns the absolute mouse state
    void getAbsoluteMouseState800600(int* x, int* y);

private:

    // Constructor is private in this singleton
    Mouse();
};

#endif // MOUSE_H

