// ============================================================================
// These functions are taken from Telemachos' PXDTUT10.
// ============================================================================
//
// Desc: A collection of vector math related functions.
//
// Copyright (C) 2000 Telemachos of Peroxide
// www.peroxide.dk

#ifndef VECTOR_MATH_H
#define VECTOR_MATH_H

#include "mtxlib.h"

// ----------------------------------------------------------------------
// Name  : closestPointOnLine()
// Input : a - first end of line segment
//         b - second end of line segment
//         p - point we wish to find closest point on line from
// Notes : Helper function for closestPointOnTriangle()
// Return: closest point on line segment
// -----------------------------------------------------------------------
vector3 closestPointOnLine(const vector3 &a, const vector3 &b, const vector3 &p);

// ----------------------------------------------------------------------
// Name  : closestPointOnTriangle()
// Input : a - first vertex in triangle
//         b - second vertex in triangle
//         c - third vertex in triangle
//         p - point we wish to find closest point on triangle from
// Notes :
// Return: closest point on line triangle edge
// -----------------------------------------------------------------------
vector3 closestPointOnTriangle(const vector3& a, const vector3& b, const vector3& c, const vector3& p);

// ----------------------------------------------------------------------
// Name  : intersectRayPlane()
// Input : rOrigin - origin of ray in world space
//         rVector - vector describing direction of ray in world space
//         pOrigin - Origin of plane
//         pNormal - Normal to plane
// Notes : Normalized directional vectors expected
// Return: distance to plane in world units, -1 if no intersection.
// -----------------------------------------------------------------------
float intersectRayPlane(const vector3 &rOrigin, const vector3 &rVector, const vector3 &pOrigin, const vector3 &pNormal);

// ----------------------------------------------------------------------
// Name  : CheckPointInTriangle()
// Input : point - point we wish to check for inclusion
//         a - first vertex in triangle
//         b - second vertex in triangle
//         c - third vertex in triangle
// Notes : Triangle should be defined in clockwise order a,b,c
// Return: TRUE if point is in triangle, FALSE if not.
// -----------------------------------------------------------------------
bool CheckPointInTriangle(const vector3 &point, const vector3 &a, const vector3 &b, const vector3 &c);

// ----------------------------------------------------------------------
// Name  : closestPointOnTriangleSurface()
// Input : a - first vertex in triangle
//         b - second vertex in triangle
//         c - third vertex in triangle
//         p - point we wish to find closest point on triangle from
//         closest - resulting point. The closest point to p on the triangles surface
// Notes :
// Return: true if closest point is valid. false if invalid.
// -----------------------------------------------------------------------
bool closestPointOnTriangleSurface(const vector3 &a, const vector3 &b, const vector3 &c,
                                        const vector3 &p, vector3 &closest, float radius);

// Return: true if ray intersects triangle
bool intersectRayTriangle(const vector3 &a, const vector3 &b, const vector3 &c,
                            const vector3 &rOrigin, const vector3 &rVector, vector3& contact);

#endif // VECTOR_MATH_H
