// Slickworm
// Copyright (c) 2002-2004, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "audiosystemopenal.h"
#include "vorbis_loader.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <vorbis/vorbisfile.h>
#include <sstream>
#include <iostream>

#if defined(USE_SOUND)

char pcmout[4096];

void load_vorbis(std::string filename, unsigned char* buffer,
                int& buffer_size, int& frequency, bool& stereo)
{
    FILE* infile;
    if ((infile = fopen(filename.c_str(), "rb")) == NULL)
    {
        std::stringstream error;
        error << "Can't open " << filename;
        throw std::string(error.str());
    }

    OggVorbis_File vf;
    int eof = 0;
    int current_section;

    if (ov_open(infile, &vf, NULL, 0) < 0)
    {
        throw std::string("Input does not appear to be an Ogg bitstream");
    }

    {
        char **ptr=ov_comment(&vf, -1)->user_comments;
        vorbis_info *vi=ov_info(&vf, -1);
        while(*ptr)
        {
            fprintf(stderr,"%s\n",*ptr);
            ++ptr;
        }
/*
        fprintf(stderr,"Bitstream is %d channel, %ldHz\n", vi->channels, vi->rate);
        fprintf(stderr,"Decoded length: %ld samples\n", (long)ov_pcm_total(&vf,-1));
        fprintf(stderr,"Encoded by: %s\n",ov_comment(&vf,-1)->vendor);
*/
        frequency = vi->rate;
        stereo = (vi->channels == 1) ? false : true;
    }

    int offset = 0;
    
    while (!eof)
    {
        long ret=ov_read(&vf, pcmout, sizeof(pcmout), 0, 2, 1, &current_section);
        if (ret == 0)
        {
            // EOF
            eof = 1;
        }
        else if (ret < 0)
        {
            // error in the stream.  Not a problem, just reporting it in
            // case we (the app) cares.  In this case, we don't.
            std::cout << "Error in ogg vorbis stream" << std::endl;
            
            if (ret == OV_HOLE)     std::cout << "OV_HOLE"      << std::endl;
            if (ret == OV_EBADLINK) std::cout << "OV_EBADLINK"  << std::endl;
        }
        else
        {
            if (offset + ret > buffer_size)
            {
                // Avoid writing beyond the end of the buffer
                std::cout << "buffer was too short to hold vorbis sample" << std::endl;
            }
            else
            {
                memcpy(buffer + offset, pcmout, ret);
                offset += ret;
            }
        }
    }

    buffer_size = offset;

    ov_clear(&vf);
}

#endif // defined(USE_SOUND)



